import json
import asyncio
import socket
from aiohttp import ClientSession
from concurrent.futures import ThreadPoolExecutor

def reverse_dns_lookup_sync(ip):
    """
    Perform a reverse DNS lookup (synchronous, for multithreading).
    """
    try:
        hostname = socket.gethostbyaddr(ip)[0]
        return hostname
    except socket.herror:
        return None

async def reverse_dns_lookup(ip, executor):
    """
    Perform a reverse DNS lookup using a thread pool.
    """
    print(f"Performing DNS lookup for IP: {ip}")
    loop = asyncio.get_event_loop()
    hostname = await loop.run_in_executor(executor, reverse_dns_lookup_sync, ip)
    if hostname:
        print(f"DNS lookup successful for IP {ip}: {hostname}")
    else:
        print(f"DNS lookup failed for IP: {ip}")
    return hostname

async def fetch_website_info(session, ip, hostname):
    """
    Fetch website title and description from the IP.
    """
    print(f"Fetching website info for IP: {ip} (Hostname: {hostname or 'Unknown'})")
    endpoints = [f"http://{ip}", f"http://{hostname}" if hostname else None]
    for url in filter(None, endpoints):
        try:
            print(f"Trying URL: {url}")
            async with session.get(url, timeout=10) as response:
                if response.status == 200:
                    print(f"Successfully fetched data from {url}")
                    html = await response.text()
                    title = (
                        html.split("<title>")[1].split("</title>")[0]
                        if "<title>" in html and "</title>" in html else "No Title"
                    )
                    description = "No Description"
                    if '<meta name="description"' in html:
                        desc_split = html.split('<meta name="description"')[1]
                        description = desc_split.split('content="')[1].split('"')[0]
                    return {"title": title, "description": description}
        except Exception as e:
            print(f"Failed to fetch from {url}: {e}")
            continue
    return {"title": "No Title", "description": "No Description"}

async def analyze_ip(ip, session, executor, results):
    """
    Analyze a single IP address to fetch hostname and website info.
    """
    hostname = await reverse_dns_lookup(ip, executor)
    website_info = await fetch_website_info(session, ip, hostname)
    result = {"ip": ip, "hostname": hostname or "Unknown", **website_info}
    results.append(result)
    print(f"Analysis complete for IP: {ip}")
    
    # Save results to file after each IP is analyzed
    with open("output.json", "w") as file:
        json.dump(results, file, indent=4)
    print("Progress saved to 'output.json'")
    return result

async def main():
    # Load IPs from the JSON file
    with open("ips_up.json", "r") as file:
        ips = json.load(file)

    results = []  # Store results incrementally
    executor = ThreadPoolExecutor()  # ThreadPool for DNS lookups
    async with ClientSession() as session:
        tasks = []
        for ip in ips:
            print(f"Starting analysis for IP: {ip}")
            tasks.append(analyze_ip(ip, session, executor, results))
        await asyncio.gather(*tasks)

    print("Analysis complete. Final results saved to 'output.json'.")

if __name__ == "__main__":
    asyncio.run(main())